@@ -8,10 +8,9 @@
 //!
 //! ```toml
 //! [dependencies.libftd2xx]
-//! version = "~0.29.0"
+//! version = "0.32"
 //! # statically link the vendor library, defaults to dynamic if not set
-//! # this will make things "just work" on Linux
-//! # not recommended on Windows due to legacy library requirements
+//! # this will make things "just work" on Linux and Windows
 //! features = ["static"]
 //! ```
 //!
@@ -59,21 +58,15 @@
 //! The shared object `libftd2xx.so` must exist on your system.
 //! See [FTDI Drivers Installation Guide for Linux] for instructions.
 //!
-//! ### Static Linking on Linux
-//!
-//! No special considerations are needed, the static library is distributed with
-//! permission from FTDI in the [libftd2xx-ffi] crate.
-//!
 //! ### Dynamic Linking on Windows
 //!
 //! The FTD2XX DLL must exist on your system PATH.
 //! The easiest way to install this is with the vendor provided [setup executable].
 //!
-//! ### Static Linking on Windows
+//! ### Static Linking on Linux or Windows
 //!
-//! You must set the "LIBMSVC_PATH" environment variable to link with
-//! `legacy_stdio_definitions.lib` (vendor library requirement).
-//! See [libftd2xx-ffi] for more information.
+//! No special considerations are needed, the static library is distributed with
+//! permission from FTDI in the [libftd2xx-ffi] crate.
 //!
 //! # References
 //!
@@ -93,23 +86,19 @@
 //! [D2xx Programmers Guide V1.4]: https://ftdichip.com/document/programming-guides/
 //! [FTDI D2XX drivers]: https://www.ftdichip.com/Drivers/D2XX.htm
 //! [FTDI Drivers Installation Guide for Linux]: http://www.ftdichip.cn/Support/Documents/AppNotes/AN_220_FTDI_Drivers_Installation_Guide_for_Linux.pdf
-//! [libftd2xx-ffi]: https://github.com/newAM/libftd2xx-ffi-rs
+//! [libftd2xx-ffi]: https://github.com/ftdi-rs/libftd2xx-ffi
 //! [setup executable]: https://www.ftdichip.com/Drivers/CDM/CDM21228_Setup.zip
 //! [udev]: https://en.wikipedia.org/wiki/Udev
 //! [`ftd2xx-embedded-hal`]: https://crates.io/crates/ftd2xx-embedded-hal
 //! [`embedded-hal`]: https://crates.io/crates/embedded-hal
 #![cfg_attr(docsrs, feature(doc_cfg))]
-#![doc(html_root_url = "https://docs.rs/libftd2xx/0.29.0")]
 #![deny(missing_docs)]
 
 mod errors;
 pub use errors::{DeviceTypeError, EepromStringsError, EepromValueError, FtStatus, TimeoutError};
 
 mod mpsse;
-pub use mpsse::{
-    ClockBits, ClockBitsIn, ClockBitsOut, ClockData, ClockDataIn, ClockDataOut, FtdiMpsse,
-    Ftx232hMpsse, MpsseCmdBuilder, MpsseSettings,
-};
+pub use mpsse::{FtdiMpsse, Ftx232hMpsse};
 
 mod types;
 use types::{vid_pid_from_id, STRING_LEN};
@@ -122,6 +111,11 @@ pub use types::{
 mod util;
 use util::slice_into_string;
 
+pub use ftdi_mpsse::{
+    ClockBits, ClockBitsIn, ClockBitsOut, ClockData, ClockDataIn, ClockDataOut, MpsseCmd,
+    MpsseCmdBuilder, MpsseCmdExecutor, MpsseSettings,
+};
+
 use libftd2xx_ffi::{
     FT_Close, FT_ClrDtr, FT_ClrRts, FT_CreateDeviceInfoList, FT_EEPROM_Program, FT_EEPROM_Read,
     FT_EE_UARead, FT_EE_UASize, FT_EE_UAWrite, FT_EraseEE, FT_GetBitMode, FT_GetDeviceInfo,
@@ -139,7 +133,7 @@ use libftd2xx_ffi::{
 #[cfg(target_os = "windows")]
 use libftd2xx_ffi::{FT_CyclePort, FT_GetComPortNumber, FT_Rescan, FT_ResetPort};
 
-#[cfg(any(target_os = "linux", target_os = "mac"))]
+#[cfg(any(target_os = "linux", target_os = "macos"))]
 use libftd2xx_ffi::{FT_GetVIDPID, FT_SetVIDPID};
 
 use log::trace;
@@ -242,8 +236,6 @@ pub fn set_vid_pid(vid: u16, pid: u16) -> Result<(), FtStatus> {
 /// println!("PID: 0x{:04X}", vid);
 /// # Ok::<(), libftd2xx::FtStatus>(())
 /// ```
-///
-/// [`set_vid_pid`]: ./fn.set_vid_pid.html
 #[cfg(any(unix, doc))]
 #[cfg_attr(docsrs, doc(cfg(unix)))]
 pub fn vid_pid() -> Result<(u32, u32), FtStatus> {
@@ -527,7 +519,7 @@ pub struct Ftdi {
 /// ```
 #[derive(Debug)]
 pub struct Ft232h {
-    handle: FT_HANDLE,
+    ftdi: Ftdi,
 }
 
 /// FT2232H device.
@@ -545,7 +537,7 @@ pub struct Ft232h {
 /// ```
 #[derive(Debug)]
 pub struct Ft2232h {
-    handle: FT_HANDLE,
+    ftdi: Ftdi,
 }
 
 /// FT4232H device.
@@ -563,7 +555,7 @@ pub struct Ft2232h {
 /// ```
 #[derive(Debug)]
 pub struct Ft4232h {
-    handle: FT_HANDLE,
+    ftdi: Ftdi,
 }
 
 /// FTD2XX functions common to all devices.
@@ -584,7 +576,7 @@ pub trait FtdiCommon {
     /// vendor driver that also return this code.
     ///
     /// This is not a native function in `libftd2xx`, this works around a bug in
-    /// `libftd2xx`, see https://github.com/newAM/libftd2xx-rs/pull/37 for more
+    /// `libftd2xx`, see <https://github.com/ftdi-rs/libftd2xx/pull/37> for more
     /// information.
     ///
     /// # Example
@@ -724,7 +716,7 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`set_usb_parameters`]: #method.set_usb_parameters
+    /// [`set_usb_parameters`]: FtdiCommon::set_usb_parameters
     fn set_usb_parameters(&mut self, in_transfer_size: u32) -> Result<(), FtStatus> {
         const MAX: u32 = 64 * 1024;
         const MIN: u32 = 64;
@@ -1186,7 +1178,6 @@ pub trait FtdiCommon {
     /// [Bit Mode Functions for the FT2232]: https://www.ftdichip.com/Support/Documents/AppNotes/AN2232C-02_FT2232CBitMode.pdf
     /// [FT232B/FT245B Bit Bang Mode]: https://www.ftdichip.com/Support/Documents/AppNotes/AN232B-01_BitBang.pdf
     /// [FTDI website]: https://www.ftdichip.com/Support/Documents/AppNotes.htm
-    /// [`BitMode`]: ./enum.BitMode.html
     ///
     /// # Example
     ///
@@ -1225,8 +1216,7 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`set_bit_mode`]: #method.set_bit_mode
-    /// [`BitMode`]: ./enum.BitMode.html
+    /// [`set_bit_mode`]: FtdiCommon::set_bit_mode
     fn bit_mode(&mut self) -> Result<u8, FtStatus> {
         let mut mode: u8 = 0;
         trace!("FT_GetBitMode({:?}, _)", self.handle());
@@ -1348,7 +1338,7 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`read_all`]: #method.read_all
+    /// [`read_all`]: FtdiCommon::read_all
     fn read(&mut self, buf: &mut [u8]) -> Result<usize, FtStatus> {
         let mut bytes_returned: u32 = 0;
         let len: u32 = u32::try_from(buf.len()).unwrap();
@@ -1426,10 +1416,10 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::TimeoutError>(())
     /// ```
     ///
-    /// [`read_all`]: #method.read_all
-    /// [`queue_status`]: #method.queue_status
-    /// [`set_timeouts`]: #method.set_timeouts
-    /// [`TimeoutError`]: ./enum.TimeoutError.html
+    /// [`read_all`]: FtdiCommon::read_all
+    /// [`queue_status`]: FtdiCommon::queue_status
+    /// [`set_timeouts`]: FtdiCommon::set_timeouts
+    /// [`TimeoutError`]: TimeoutError
     fn read_all(&mut self, buf: &mut [u8]) -> Result<(), TimeoutError> {
         let num_read = self.read(buf)?;
         if num_read != buf.len() {
@@ -1814,7 +1804,7 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`eeprom_user_size`]: #method.eeprom_user_size
+    /// [`eeprom_user_size`]: FtdiCommon::eeprom_user_size
     fn eeprom_user_read(&mut self, buf: &mut [u8]) -> Result<usize, FtStatus> {
         let mut num_read: u32 = 0;
         let len: u32 = u32::try_from(buf.len()).unwrap();
@@ -1841,7 +1831,7 @@ pub trait FtdiCommon {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`eeprom_user_size`]: #method.eeprom_user_size
+    /// [`eeprom_user_size`]: FtdiCommon::eeprom_user_size
     fn eeprom_user_write(&mut self, buf: &[u8]) -> Result<(), FtStatus> {
         let len: u32 = u32::try_from(buf.len()).unwrap();
         trace!("FT_EE_UAWrite({:?}, _, {})", self.handle(), len);
@@ -1853,7 +1843,7 @@ pub trait FtdiCommon {
 
 /// FTDI device-specific EEPROM trait.
 pub trait FtdiEeprom<
-    T: Default + std::fmt::Debug + std::convert::From<Eeprom>,
+    T: std::fmt::Debug + std::convert::From<Eeprom>,
     Eeprom: Default + std::fmt::Debug + std::convert::From<T>,
 >: FtdiCommon
 {
@@ -2001,8 +1991,8 @@ impl Ftdi {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`with_index`]: #method.with_index
-    /// [`with_serial_number`]: #method.with_serial_number
+    /// [`with_index`]: Ftdi::with_index
+    /// [`with_serial_number`]: Ftdi::with_serial_number
     pub fn new() -> Result<Ftdi, FtStatus> {
         Ftdi::with_index(0)
     }
@@ -2026,7 +2016,7 @@ impl Ftdi {
     /// # Ok::<(), libftd2xx::FtStatus>(())
     /// ```
     ///
-    /// [`with_serial_number`]: #method.with_serial_number
+    /// [`with_serial_number`]: Ftdi::with_serial_number
     pub fn with_index(index: i32) -> Result<Ftdi, FtStatus> {
         let mut handle: FT_HANDLE = std::ptr::null_mut();
         trace!("FT_Open({}, _)", index);
@@ -2084,7 +2074,9 @@ impl Ft232h {
     /// ```
     pub unsafe fn with_serial_number_unchecked(serial_number: &str) -> Result<Ft232h, FtStatus> {
         let handle = ft_open_ex(serial_number, FT_OPEN_BY_SERIAL_NUMBER)?;
-        Ok(Ft232h { handle })
+        Ok(Ft232h {
+            ftdi: Ftdi { handle },
+        })
     }
 
     /// Open a `Ft232h` device and initialize the handle.
@@ -2135,7 +2127,9 @@ impl Ft2232h {
     /// ```
     pub unsafe fn with_serial_number_unchecked(serial_number: &str) -> Result<Ft2232h, FtStatus> {
         let handle = ft_open_ex(serial_number, FT_OPEN_BY_SERIAL_NUMBER)?;
-        Ok(Ft2232h { handle })
+        Ok(Ft2232h {
+            ftdi: Ftdi { handle },
+        })
     }
 
     /// Open a `Ft2232h` device and initialize the handle.
@@ -2186,7 +2180,9 @@ impl Ft4232h {
     /// ```
     pub unsafe fn with_serial_number_unchecked(serial_number: &str) -> Result<Ft4232h, FtStatus> {
         let handle = ft_open_ex(serial_number, FT_OPEN_BY_SERIAL_NUMBER)?;
-        Ok(Ft4232h { handle })
+        Ok(Ft4232h {
+            ftdi: Ftdi { handle },
+        })
     }
 
     /// Open a `Ft4232h` device and initialize the handle.
@@ -2226,19 +2222,41 @@ impl FtdiCommon for Ftdi {
     }
 }
 
+impl Drop for Ftdi {
+    fn drop(&mut self) {
+        self.close().ok();
+    }
+}
+
 macro_rules! impl_boilerplate_for {
     ($DEVICE:ident, $TYPE:expr) => {
         impl FtdiCommon for $DEVICE {
             const DEVICE_TYPE: DeviceType = $TYPE;
 
             fn handle(&mut self) -> FT_HANDLE {
-                self.handle
+                self.ftdi.handle
             }
 
             fn device_type(&mut self) -> Result<DeviceType, FtStatus> {
                 Ok(Self::DEVICE_TYPE)
             }
         }
+
+        impl MpsseCmdExecutor for $DEVICE {
+            type Error = TimeoutError;
+
+            fn init(&mut self, settings: &MpsseSettings) -> Result<(), Self::Error> {
+                self.initialize_mpsse(settings)
+            }
+
+            fn send(&mut self, data: &[u8]) -> Result<(), Self::Error> {
+                self.write_all(data)
+            }
+
+            fn recv(&mut self, data: &mut [u8]) -> Result<(), Self::Error> {
+                self.read_all(data)
+            }
+        }
     };
 }
 
@@ -2255,9 +2273,7 @@ macro_rules! impl_try_from_for {
                         detected: device_type,
                     })
                 } else {
-                    Ok($DEVICE {
-                        handle: ft.handle(),
-                    })
+                    Ok($DEVICE { ftdi: ft })
                 }
             }
         }
